#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <math.h>
#include "scale.h"

static double pdSet[] = {1.0,2.0,5.0,10.0};
#define SET_LEN (sizeof(pdSet)/sizeof(double)-1)

static double scFirstNiceNum (double,int *,double *);
static double scNextNiceNum (double *,int,int *,double *);
static void scCalcExtLabel (double,double,double,int *,int *);
static double scPower (double,int);

void scMaxInterval(dDataMin,dDataMax,nMaxIntervals,pdScaleMin,pdScaleMax,pnActual)
double dDataMin,dDataMax,*pdScaleMin,*pdScaleMax;
int nMaxIntervals,*pnActual;
{
  double dIntervalSize,dPowerOfTen,dNiceNum;
  int iIndex,nLoMult,nHiMult;

  assert(dDataMin < dDataMax);
  assert(nMaxIntervals >= 2);

  dIntervalSize = (dDataMax - dDataMin)/nMaxIntervals;

  for(dNiceNum = scFirstNiceNum(dIntervalSize,&iIndex,&dPowerOfTen);
       dNiceNum < dIntervalSize;
       dNiceNum = scNextNiceNum (pdSet,SET_LEN,&iIndex,&dPowerOfTen))
  {
    ;
  }

  scCalcExtLabel(dDataMin,dDataMax,dNiceNum,&nLoMult,&nHiMult);

  while(nHiMult - nLoMult > nMaxIntervals)
  {
       dNiceNum = scNextNiceNum(pdSet,SET_LEN,&iIndex,&dPowerOfTen);
       scCalcExtLabel(dDataMin,dDataMax,dNiceNum,&nLoMult,&nHiMult);
  }

  *pdScaleMin = nLoMult*dNiceNum;
  *pdScaleMax = nHiMult*dNiceNum;
  *pnActual = nHiMult - nLoMult;
}

double scFirstNiceNum(dIntervalSize,piIndex,pdPowerOfTen)
double dIntervalSize,*pdPowerOfTen;
int *piIndex;
{
  int iExponent;

  iExponent = (int)floor(log10(dIntervalSize));
  *pdPowerOfTen = scPower(10.0,iExponent);
  if(*pdPowerOfTen*10.0 <= dIntervalSize)
   {
    *pdPowerOfTen *= 10.0;
   }

   *piIndex = 0;

   return(*pdPowerOfTen);
}

double scNextNiceNum(pdSet,nSet,piIndex,pdPowerOfTen)
double *pdSet,*pdPowerOfTen;
int nSet,*piIndex;
{
  (*piIndex)++;

  if(*piIndex >= nSet)
   {
     *piIndex = 0;
     *pdPowerOfTen *= 10.0;
   }

  return(pdSet[*piIndex]* *pdPowerOfTen);
}

void scCalcExtLabel(dDataMin,dDataMax,dNiceNum,pnLoMult,pnHiMult)
double dDataMin,dDataMax,dNiceNum;
int *pnLoMult,*pnHiMult;
{
  *pnLoMult = (int)floor(dDataMin/dNiceNum);

  if((double)(*pnLoMult + 1) * dNiceNum <= dDataMin)
   {
     (*pnLoMult)++;
   }

  *pnHiMult = (int) ceil(dDataMax/dNiceNum);
  if((double) (*pnHiMult - 1)*dNiceNum >= dDataMax)
   {
     (*pnHiMult)--;
   }
}

static double scPower (dRoot,iExponent)
double dRoot;
int iExponent;
{
  double dResult;

  if(iExponent < 0)
   {
     dRoot = 1.0/dRoot;
     iExponent = -iExponent;
   }

  dResult = 1.0;
  while(iExponent)
   {
     if(iExponent & 1)
      {
	dResult *= dRoot;
      }

     iExponent >>= 1;

     if(iExponent)
      {
	dRoot *= dRoot;
      }
   }

   return(dResult);
}
